@@ -1,16 +1,23 @@
 #ifndef SW_ATOMIC_H_
 #define SW_ATOMIC_H_
 
-#if defined(__x86_64__)
-#define SW_ATOMIC_64_LEN                     (sizeof("-9223372036854775808") - 1)
-typedef volatile int64_t atomic_int64_t;
-typedef volatile uint64_t atomic_uint64_t;
+typedef volatile int32_t                  sw_atomic_int32_t;
+typedef volatile uint32_t                 sw_atomic_uint32_t;
+
+#ifdef __x86_64__
+typedef volatile int64_t                  sw_atomic_int64_t;
+typedef volatile uint64_t                 sw_atomic_uint64_t;
 #endif
 
-#define SW_ATOMIC_32_LEN                      (sizeof("-2147483648") - 1)
-typedef volatile int32_t atomic_int32_t;
-typedef volatile uint32_t atomic_uint32_t;
-typedef atomic_uint32_t  sw_atomic_t;
+#ifdef __x86_64__
+typedef sw_atomic_int64_t                 sw_atomic_long_t;
+typedef sw_atomic_uint64_t                sw_atomic_ulong_t;
+#else
+typedef sw_atomic_int32_t                 sw_atomic_long_t;
+typedef sw_atomic_uint32_t                sw_atomic_ulong_t;
+#endif
+
+typedef sw_atomic_uint32_t                sw_atomic_t;
 
 #define sw_atomic_cmp_set(lock, old, set) __sync_bool_compare_and_swap(lock, old, set)
 #define sw_atomic_fetch_add(value, add)   __sync_fetch_and_add(value, add)
