@@ -499,7 +499,7 @@ struct _swServer
 
     int *cpu_affinity_available;
     int cpu_affinity_available_num;
-    
+
     double send_timeout;
 
     uint16_t listen_port_num;
@@ -574,8 +574,6 @@ struct _swServer
     int stream_fd;
     swProtocol stream_protocol;
     swLinkedList *buffer_pool;
-    int last_stream_fd;
-    int last_session_id;
 
 #ifdef SW_BUFFER_RECV_TIME
     double last_receive_usec;
