@@ -1,3 +1,19 @@
+/*
+  +----------------------------------------------------------------------+
+  | Swoole                                                               |
+  +----------------------------------------------------------------------+
+  | This source file is subject to version 2.0 of the Apache license,    |
+  | that is bundled with this package in the file LICENSE, and is        |
+  | available through the world-wide-web at the following url:           |
+  | http://www.apache.org/licenses/LICENSE-2.0.html                      |
+  | If you did not receive a copy of the Apache2.0 license and are unable|
+  | to obtain it through the world-wide-web, please send a note to       |
+  | license@swoole.com so we can mail you a copy immediately.            |
+  +----------------------------------------------------------------------+
+  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  +----------------------------------------------------------------------+
+*/
+
 #pragma once
 
 #include "swoole.h"
@@ -38,15 +54,21 @@ public:
     bool listen(int backlog = 0);
     bool sendfile(char *filename, off_t offset, size_t length);
     ssize_t sendto(char *address, int port, char *data, int len);
-    ssize_t recvfrom(void *__buf, size_t __n, char *address, int *port = nullptr);
+    ssize_t recvfrom(void *__buf, size_t __n);
+    ssize_t recvfrom(void *__buf, size_t __n, struct sockaddr *_addr, socklen_t *_socklen);
     swString* get_buffer();
 
-    void setTimeout(double timeout)
+    inline void setTimeout(double timeout)
     {
         _timeout = timeout;
     }
 
-    int get_fd()
+    inline void set_timeout(struct timeval *timeout)
+    {
+        setTimeout((double) timeout->tv_sec + ((double) timeout->tv_usec / 1000 / 1000));
+    }
+
+    inline int get_fd()
     {
         return socket->fd;
     }
@@ -54,6 +76,7 @@ public:
 #ifdef SW_USE_OPENSSL
     bool ssl_handshake();
     int ssl_verify(bool allow_self_signed);
+    bool ssl_accept();
 #endif
 
 protected:
@@ -82,6 +105,7 @@ protected:
 
         buffer = nullptr;
         protocol = {0};
+        bind_address_info = {{}, 0};
 
         protocol.package_length_type = 'N';
         protocol.package_length_size = 4;
@@ -89,8 +113,7 @@ protected:
         protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
 
 #ifdef SW_USE_OPENSSL
-        open_ssl = 0;
-        ssl_wait_handshake = 0;
+        open_ssl = false;
         ssl_context = NULL;
         ssl_option = {0};
 #endif
@@ -150,13 +173,14 @@ public:
 
     swProtocol protocol;
     swString *buffer;
+    swSocketAddress bind_address_info;
 
     struct _swSocks5 *socks5_proxy;
     struct _http_proxy* http_proxy;
 
 #ifdef SW_USE_OPENSSL
-    uint8_t open_ssl :1;
-    uint8_t ssl_wait_handshake :1;
+    bool open_ssl;
+    bool ssl_wait_handshake;
     SSL_CTX *ssl_context;
     swSSL_option ssl_option;
 #endif
