@@ -37,6 +37,7 @@ extern "C" {
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -609,10 +610,6 @@ typedef struct _swConnection
     double last_time_usec;
 #endif
 
-#ifdef SW_USE_TIMEWHEEL
-    uint16_t timewheel_index;
-#endif
-
     /**
      * bind uid
      */
@@ -1479,12 +1476,6 @@ struct _swReactor
     time_t last_malloc_trim_time;
 #endif
 
-#ifdef SW_USE_TIMEWHEEL
-    swTimeWheel *timewheel;
-    uint16_t heartbeat_interval;
-    time_t last_heartbeat_time;
-#endif
-
     /**
      * for thread
      */
@@ -1992,6 +1983,7 @@ struct _swTimer_node
     swTimerCallback callback;
     int64_t exec_msec;
     uint32_t interval;
+    uint64_t round;
     long id;
     int type;                 //0 normal node 1 node for client_coro
     uint8_t remove;
@@ -2015,6 +2007,7 @@ struct _swTimer
     int use_pipe;
     int lasttime;
     int fd;
+    uint64_t round;
     long _next_id;
     long _current_id;
     long _next_msec;
