@@ -34,6 +34,7 @@ extern "C"
 #define SW_WEBSOCKET_MASKED(frm) (frm->header.MASK)
 #define SW_WEBSOCKET_CLOSE_CODE_LEN         2
 #define SW_WEBSOCKET_CLOSE_REASON_MAX_LEN   125
+#define SW_WEBSOCKET_OPCODE_MAX  WEBSOCKET_OPCODE_PONG
 
 #define FRAME_SET_FIN(BYTE) (((BYTE) & 0x01) << 7)
 #define FRAME_SET_OPCODE(BYTE) ((BYTE) & 0x0F)
@@ -94,8 +95,9 @@ enum swWebsocketCode
 };
 
 int swWebSocket_get_package_length(swProtocol *protocol, swConnection *conn, char *data, uint32_t length);
-void swWebSocket_encode(swString *buffer, char *data, size_t length, char opcode, int finish, int mask);
+void swWebSocket_encode(swString *buffer, char *data, size_t length, char opcode, uint8_t finish, uint8_t mask);
 void swWebSocket_decode(swWebSocket_frame *frame, swString *data);
+int swWebSocket_pack_close_frame(swString *buffer, int code, char* reason, size_t length, uint8_t mask);
 void swWebSocket_print_frame(swWebSocket_frame *frame);
 int swWebSocket_dispatch_frame(swConnection *conn, char *data, uint32_t length);
 
