@@ -67,7 +67,7 @@ BEGIN_EXTERN_C()
 #include <ext/standard/basic_functions.h>
 #include <ext/standard/php_http.h>
 
-#define PHP_SWOOLE_VERSION  "4.1.2"
+#define PHP_SWOOLE_VERSION  "4.2.1"
 #define PHP_SWOOLE_CHECK_CALLBACK
 #define PHP_SWOOLE_ENABLE_FASTCALL
 #define PHP_SWOOLE_CLIENT_USE_POLL
@@ -410,7 +410,6 @@ void swoole_process_init(int module_number TSRMLS_DC);
 void swoole_process_pool_init(int module_number TSRMLS_DC);
 void swoole_http_server_init(int module_number TSRMLS_DC);
 #ifdef SW_USE_HTTP2
-void swoole_http2_client_init(int module_number TSRMLS_DC);
 #ifdef SW_COROUTINE
 void swoole_http2_client_coro_init(int module_number TSRMLS_DC);
 #endif
@@ -446,7 +445,8 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC);
 #ifdef SW_USE_OPENSSL
 void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset TSRMLS_DC);
 #endif
-void php_swoole_websocket_unpack(swString *data, zval *zframe TSRMLS_DC);
+void php_swoole_websocket_frame_unpack(swString *data, zval *zframe TSRMLS_DC);
+int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opcode, zend_bool fin, zend_bool mask);
 void php_swoole_sha1(const char *str, int _len, unsigned char *digest);
 int php_swoole_client_isset_callback(zval *zobject, int type TSRMLS_DC);
 
@@ -484,7 +484,7 @@ php_socket *swoole_convert_to_socket(int sock);
 void php_swoole_server_before_start(swServer *serv, zval *zobject TSRMLS_DC);
 void php_swoole_server_send_yield(swServer *serv, int fd, zval *zdata, zval *return_value);
 void php_swoole_get_recv_data(zval *zdata, swEventData *req, char *header, uint32_t header_length);
-int php_swoole_get_send_data(zval *zdata, char **str TSRMLS_DC);
+ssize_t php_swoole_get_send_data(zval *zdata, char **str TSRMLS_DC);
 void php_swoole_onConnect(swServer *, swDataHead *);
 int php_swoole_onReceive(swServer *, swEventData *);
 int php_swoole_onPacket(swServer *, swEventData *);
@@ -660,6 +660,12 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
 # define ZEND_HASH_APPLY_PROTECTION(p) 1
 #endif
 
+/* PHP 7.4 forward compatibility */
+#ifndef ZEND_ACC_CTOR
+#define ZEND_ACC_CTOR 0
+#define ZEND_ACC_DTOR 0
+#endif
+
 END_EXTERN_C()
 
 #endif	/* PHP_SWOOLE_H */
