@@ -1,3 +1,19 @@
+/*
+  +----------------------------------------------------------------------+
+  | Swoole                                                               |
+  +----------------------------------------------------------------------+
+  | This source file is subject to version 2.0 of the Apache license,    |
+  | that is bundled with this package in the file LICENSE, and is        |
+  | available through the world-wide-web at the following url:           |
+  | http://www.apache.org/licenses/LICENSE-2.0.html                      |
+  | If you did not receive a copy of the Apache2.0 license and are unable|
+  | to obtain it through the world-wide-web, please send a note to       |
+  | license@swoole.com so we can mail you a copy immediately.            |
+  +----------------------------------------------------------------------+
+  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  +----------------------------------------------------------------------+
+*/
+
 #include "coroutine.h"
 #include "context.h"
 #include <string>
@@ -16,12 +32,12 @@ struct coroutine_s
 public:
     Context ctx;
     int cid;
-    void *ptr;
+    void *task;
     coroutine_s(int _cid, size_t stack_size, coroutine_func_t fn, void *private_data) :
             ctx(stack_size, fn, private_data)
     {
         cid = _cid;
-        ptr = NULL;
+        task = NULL;
     }
 };
 
@@ -139,7 +155,7 @@ void coroutine_yield(coroutine_t *co)
 {
     if (swCoroG.onYield)
     {
-        swCoroG.onYield(co->ptr);
+        swCoroG.onYield(co->task);
     }
     swCoroG.call_stack_size--;
     co->ctx.SwapOut();
@@ -149,7 +165,7 @@ void coroutine_resume(coroutine_t *co)
 {
     if (swCoroG.onResume)
     {
-        swCoroG.onResume(co->ptr);
+        swCoroG.onResume(co->task);
     }
     swCoroG.call_stack[swCoroG.call_stack_size++] = co;
     co->ctx.SwapIn();
@@ -187,12 +203,12 @@ void coroutine_release(coroutine_t *co)
     delete co;
 }
 
-void coroutine_set_ptr(coroutine_t *co, void *ptr)
+void coroutine_set_task(coroutine_t *co, void *task)
 {
-    co->ptr = ptr;
+    co->task = task;
 }
 
-void* coroutine_get_ptr_by_cid(int cid)
+void* coroutine_get_task_by_cid(int cid)
 {
     coroutine_t *co = swCoroG.coroutines[cid];
     if (co == nullptr)
@@ -201,7 +217,7 @@ void* coroutine_get_ptr_by_cid(int cid)
     }
     else
     {
-        return co->ptr;
+        return co->task;
     }
 }
 
@@ -210,14 +226,27 @@ coroutine_t* coroutine_get_by_id(int cid)
     return swCoroG.coroutines[cid];
 }
 
-coroutine_t* coroutine_get_current_task()
+coroutine_t* coroutine_get_current()
 {
     return (swCoroG.call_stack_size > 0) ? swCoroG.call_stack[swCoroG.call_stack_size - 1] : nullptr;
 }
 
+void* coroutine_get_current_task()
+{
+    coroutine_t* co = coroutine_get_current();
+    if (co == nullptr)
+    {
+        return nullptr;
+    }
+    else
+    {
+        return co->task;
+    }
+}
+
 int coroutine_get_current_cid()
 {
-    coroutine_t* co = coroutine_get_current_task();
+    coroutine_t* co = coroutine_get_current();
     if (co)
     {
         return co->cid;
@@ -263,3 +292,9 @@ void coroutine_set_onClose(coro_php_close_t func)
 {
     swCoroG.onClose = func;
 }
+
+void coroutine_set_stack_size(int stack_size)
+{
+    swCoroG.stack_size = stack_size;
+}
+
