@@ -1,3 +1,19 @@
+/*
+  +----------------------------------------------------------------------+
+  | Swoole                                                               |
+  +----------------------------------------------------------------------+
+  | This source file is subject to version 2.0 of the Apache license,    |
+  | that is bundled with this package in the file LICENSE, and is        |
+  | available through the world-wide-web at the following url:           |
+  | http://www.apache.org/licenses/LICENSE-2.0.html                      |
+  | If you did not receive a copy of the Apache2.0 license and are unable|
+  | to obtain it through the world-wide-web, please send a note to       |
+  | license@swoole.com so we can mail you a copy immediately.            |
+  +----------------------------------------------------------------------+
+  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  +----------------------------------------------------------------------+
+*/
+
 #include "channel.h"
 
 using namespace swoole;
@@ -66,6 +82,10 @@ void Channel::notify(enum channel_op type)
 
 void* Channel::pop(double timeout)
 {
+    if (closed)
+    {
+        return nullptr;
+    }
     timeout_msg_t msg;
     msg.error = false;
     if (timeout > 0)
@@ -87,14 +107,14 @@ void* Channel::pop(double timeout)
     {
         yield(CONSUMER);
     }
-    if (msg.error)
-    {
-        return nullptr;
-    }
     if (msg.timer)
     {
         swTimer_del(&SwooleG.timer, msg.timer);
     }
+    if (msg.error || closed)
+    {
+        return nullptr;
+    }
     /**
      * pop data
      */
@@ -112,10 +132,18 @@ void* Channel::pop(double timeout)
 
 bool Channel::push(void *data)
 {
+    if (closed)
+    {
+        return false;
+    }
     if (is_full() || producer_queue.size() > 0)
     {
         yield(PRODUCER);
     }
+    if (closed)
+    {
+        return false;
+    }
     /**
      * push data
      */
@@ -143,7 +171,7 @@ bool Channel::close()
     {
         notify(PRODUCER);
     }
-    while (consumer_queue.size() > 0 && notify_consumer_count < producer_queue.size())
+    while (consumer_queue.size() > 0 && notify_consumer_count < consumer_queue.size())
     {
         notify(CONSUMER);
     }
