@@ -347,18 +347,18 @@ Socket::Socket(enum swSocket_type _type)
     init();
 }
 
-Socket::Socket(int _fd, Socket *sock)
+Socket::Socket(int _fd, Socket *server_sock)
 {
-    reactor = sock->reactor;
+    reactor = server_sock->reactor;
 
     socket = swReactor_get(reactor, _fd);
     bzero(socket, sizeof(swConnection));
     socket->fd = _fd;
     socket->object = this;
-    socket->socket_type = sock->type;
+    socket->socket_type = server_sock->type;
 
-    _sock_domain = sock->_sock_domain;
-    _sock_type = sock->_sock_type;
+    _sock_domain = server_sock->_sock_domain;
+    _sock_type = server_sock->_sock_type;
     init();
 }
 
@@ -512,6 +512,12 @@ bool Socket::connect(string host, int port, int flags)
     {
         return false;
     }
+#ifdef SW_USE_OPENSSL
+    if (open_ssl && ssl_handshake() == false)
+    {
+        return false;
+    }
+#endif
     //socks5 proxy
     if (socks5_proxy && socks5_handshake() == false)
     {
@@ -875,8 +881,7 @@ bool Socket::bind(std::string address, int port)
     bind_address = address;
     bind_port = port;
 
-    struct sockaddr_storage sa_storage = { 0 };
-    struct sockaddr *sock_type = (struct sockaddr*) &sa_storage;
+    struct sockaddr *sock_type = (struct sockaddr*) &bind_address_info.addr.un;
 
     int retval;
     switch (_sock_domain)
@@ -938,12 +943,27 @@ bool Socket::bind(std::string address, int port)
 
 bool Socket::listen(int backlog)
 {
+    if (backlog <= 0)
+    {
+        backlog = SW_BACKLOG;
+    }
     _backlog = backlog;
     if (::listen(socket->fd, backlog) != 0)
     {
         errCode = errno;
         return false;
     }
+#ifdef SW_USE_OPENSSL
+    if (open_ssl)
+    {
+        ssl_context = swSSL_get_context(&ssl_option);
+        if (ssl_context == nullptr)
+        {
+            swWarn("swSSL_get_context() error.");
+            return false;
+        }
+    }
+#endif
     return true;
 }
 
@@ -965,26 +985,39 @@ Socket* Socket::accept()
     }
     int conn;
     swSocketAddress client_addr;
-    socklen_t client_addrlen = sizeof(client_addr);
+    client_addr.len = sizeof(client_addr.addr);
 
 #ifdef HAVE_ACCEPT4
-    conn = ::accept4(socket->fd, (struct sockaddr *) &client_addr, &client_addrlen, SOCK_NONBLOCK | SOCK_CLOEXEC);
+    conn = ::accept4(socket->fd, (struct sockaddr *) &client_addr.addr, &client_addr.len, SOCK_NONBLOCK | SOCK_CLOEXEC);
 #else
-    conn = ::accept(socket->fd, (struct sockaddr *) &client_addr, &client_addrlen);
+    conn = ::accept(socket->fd, (struct sockaddr *) &client_addr.addr, &client_addr.len);
     if (conn >= 0)
     {
         swoole_fcntl_set_option(conn, 1, 1);
     }
 #endif
-    if (conn >= 0)
-    {
-        return new Socket(conn, this);
-    }
-    else
+    if (conn < 0)
     {
         errCode = errno;
         return nullptr;
     }
+    Socket *client_sock = new Socket(conn, this);
+    memcpy(&client_sock->socket->info.addr, &client_addr.addr, client_addr.len);
+#ifdef SW_USE_OPENSSL
+    if (open_ssl)
+    {
+        if (swSSL_create(client_sock->socket, ssl_context, 0) < 0)
+        {
+            _delete: delete client_sock;
+            return nullptr;
+        }
+        if (client_sock->ssl_accept() == false)
+        {
+            goto _delete;
+        }
+    }
+#endif
+    return client_sock;
 }
 
 string Socket::resolve(string domain_name)
@@ -997,7 +1030,14 @@ string Socket::resolve(string domain_name)
 
     swAio_event ev;
     bzero(&ev, sizeof(swAio_event));
-    ev.nbytes = SW_IP_MAX_LENGTH;
+    if (domain_name.size() < SW_IP_MAX_LENGTH)
+    {
+        ev.nbytes = SW_IP_MAX_LENGTH + 1;
+    }
+    else
+    {
+        ev.nbytes = domain_name.size() + 1;
+    }
     ev.buf = sw_malloc(ev.nbytes);
     if (!ev.buf)
     {
@@ -1010,6 +1050,7 @@ string Socket::resolve(string domain_name)
     ev.flags = _sock_domain;
     ev.type = SW_AIO_GETHOSTBYNAME;
     ev.object = this;
+    ev.handler = swAio_handler_gethostbyname;
     ev.callback = socket_onResolveCompleted;
 
     if (SwooleAIO.init == 0)
@@ -1112,18 +1153,18 @@ bool Socket::close()
 
     int fd = socket->fd;
 
-    if (_sock_type == SW_SOCK_UNIX_DGRAM)
+    if (_sock_domain == AF_UNIX && bind_address.size() > 0)
     {
-        unlink(socket->info.addr.un.sun_path);
+        unlink(bind_address_info.addr.un.sun_path);
     }
 
 #ifdef SW_USE_OPENSSL
-    if (open_ssl && ssl_context)
+    if (socket->ssl)
+    {
+        swSSL_close(socket);
+    }
+    if (ssl_context)
     {
-        if (socket->ssl)
-        {
-            swSSL_close(socket);
-        }
         swSSL_free_context(ssl_context);
         if (ssl_option.cert_file)
         {
@@ -1254,6 +1295,36 @@ bool Socket::ssl_handshake()
     return true;
 }
 
+bool Socket::ssl_accept()
+{
+    open_ssl = true;
+    while (true)
+    {
+        int retval = swSSL_accept(socket);
+        if (retval == SW_ERROR)
+        {
+            return false;
+        }
+        else if (retval == SW_READY)
+        {
+            return true;
+        }
+        /**
+         * wait event
+         */
+        int events = socket->ssl_want_write ? SW_EVENT_WRITE : SW_EVENT_READ;
+        if (!wait_events(events))
+        {
+            return false;
+        }
+        yield(SOCKET_LOCK_READ | SOCKET_LOCK_WRITE);
+        if (errCode == ETIMEDOUT)
+        {
+            return false;
+        }
+    }
+}
+
 int Socket::ssl_verify(bool allow_self_signed)
 {
     if (swSSL_verify(socket, allow_self_signed) < 0)
@@ -1365,17 +1436,21 @@ ssize_t Socket::sendto(char *address, int port, char *data, int len)
     }
 }
 
-ssize_t Socket::recvfrom(void *__buf, size_t __n, char *address, int *port)
+ssize_t Socket::recvfrom(void *__buf, size_t __n)
+{
+    socket->info.len = sizeof(socket->info.addr);
+    return recvfrom(__buf, __n, (struct sockaddr*) &socket->info.addr, &socket->info.len);
+}
+
+ssize_t Socket::recvfrom(void *__buf, size_t __n, struct sockaddr* _addr, socklen_t *_socklen)
 {
     if (read_locked)
     {
         swWarn("socket has already been bound to another coroutine.");
         return -1;
     }
-    socket->info.len = sizeof(socket->info.addr);
     ssize_t retval;
-
-    _recv: retval = ::recvfrom(socket->fd, __buf, __n, 0, (struct sockaddr *) &socket->info.addr, &socket->info.len);
+    _recv: retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
     if (retval < 0)
     {
         if (errno == EINTR)
@@ -1393,16 +1468,11 @@ ssize_t Socket::recvfrom(void *__buf, size_t __n, char *address, int *port)
             {
                 return -1;
             }
-            retval = ::recvfrom(socket->fd, __buf, __n, 0, (struct sockaddr *) &socket->info.addr, &socket->info.len);
+            retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
             if (retval < 0)
             {
                 errCode = errno;
             }
-            else
-            {
-                strcpy(address, swConnection_get_ip(socket));
-                *port = swConnection_get_port(socket);
-            }
         }
         else
         {
@@ -1423,9 +1493,13 @@ ssize_t Socket::recv_packet()
 
     if (open_length_check)
     {
-        uint32_t header_len;
-
-        _get_header_len: header_len = protocol.package_length_offset + protocol.package_length_size;
+        //unprocessed data
+        if (buffer->offset > 0)
+        {
+            memmove(buffer->str, buffer->str + buffer->offset, buffer->length);
+            buffer->offset = 0;
+        }
+        uint32_t header_len = protocol.package_length_offset + protocol.package_length_size;
         if (buffer->length > 0)
         {
             if (buffer->length < header_len)
@@ -1443,10 +1517,6 @@ ssize_t Socket::recv_packet()
         {
             return 0;
         }
-        else if (retval < 0 || retval != header_len)
-        {
-            return 0;
-        }
         else
         {
             buffer->length += retval;
@@ -1483,12 +1553,13 @@ ssize_t Socket::recv_packet()
         }
         else if ((size_t) buf_len < buffer->length)
         {
-            buffer->length = buffer->length - buf_len;
-            memmove(buffer->str, buffer->str + buf_len, buffer->length);
-            goto _get_header_len;
+            //unprocessed data
+            buffer->length -= buf_len;
+            buffer->offset = buf_len;
+            return buf_len;
         }
 
-        if ((size_t) buf_len >= buffer->size)
+        if ((size_t) buf_len > buffer->size)
         {
             if (swString_extend(buffer, buf_len) < 0)
             {
