@@ -49,10 +49,8 @@ static void swServer_signal_hanlder(int sig);
 static int swServer_start_proxy(swServer *serv);
 static void swServer_disable_accept(swReactor *reactor);
 
-#ifndef SW_USE_TIMEWHEEL
 static void swHeartbeatThread_start(swServer *serv);
 static void swHeartbeatThread_loop(swThreadParam *param);
-#endif
 
 static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int from_fd, int reactor_id);
 
@@ -368,7 +366,6 @@ static int swServer_start_proxy(swServer *serv)
         return SW_ERR;
     }
 
-#ifndef SW_USE_TIMEWHEEL
     /**
      * heartbeat thread
      */
@@ -377,7 +374,6 @@ static int swServer_start_proxy(swServer *serv)
         swTrace("hb timer start, time: %d live time:%d", serv->heartbeat_check_interval, serv->heartbeat_idle_time);
         swHeartbeatThread_start(serv);
     }
-#endif
 
     /**
      * master thread loop
@@ -1071,20 +1067,6 @@ int swServer_tcp_send(swServer *serv, int fd, void *data, uint32_t length)
     }
     else
     {
-        if (fd == serv->last_session_id && serv->last_stream_fd > 0)
-        {
-            int _l = htonl(length);
-            if (SwooleG.main_reactor->write(SwooleG.main_reactor, serv->last_stream_fd, (void *) &_l, sizeof(_l)) < 0)
-            {
-                return SW_ERR;
-            }
-            if (SwooleG.main_reactor->write(SwooleG.main_reactor, serv->last_stream_fd, data, length) < 0)
-            {
-                return SW_ERR;
-            }
-            return SW_OK;
-        }
-
         _send.info.fd = fd;
         _send.info.type = SW_EVENT_TCP;
         _send.data = data;
@@ -1098,9 +1080,8 @@ int swServer_tcp_send(swServer *serv, int fd, void *data, uint32_t length)
             _send.info.len = length;
             _send.length = 0;
         }
-        return factory->finish(factory, &_send);
+        return factory->finish(factory, &_send) < 0 ? SW_ERR : SW_OK;
     }
-    return SW_OK;
 }
 
 /**
@@ -1684,7 +1665,6 @@ static void swServer_signal_hanlder(int sig)
     }
 }
 
-#ifndef SW_USE_TIMEWHEEL
 static void swHeartbeatThread_start(swServer *serv)
 {
     swThreadParam *param;
@@ -1774,7 +1754,6 @@ static void swHeartbeatThread_loop(swThreadParam *param)
     }
     pthread_exit(0);
 }
-#endif
 
 /**
  * new connection
@@ -1796,7 +1775,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     bzero(connection, sizeof(swConnection));
 
     //TCP Nodelay
-    if (ls->open_tcp_nodelay)
+    if (ls->open_tcp_nodelay && ls->type != SW_SOCK_UNIX_STREAM)
     {
         int sockopt = 1;
         if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &sockopt, sizeof(sockopt)) < 0)
