@@ -262,18 +262,16 @@ static int swWorker_onStreamPackage(swConnection *conn, char *data, uint32_t len
     swServer *serv = SwooleG.serv;
     swEventData *task = (swEventData *) (data + 4);
 
-    serv->last_stream_fd = conn->fd;
-
     swString *package = swWorker_get_buffer(serv, task->info.from_id);
     uint32_t data_length = length - sizeof(task->info) - 4;
     //merge data to package buffer
     swString_append_ptr(package, data + sizeof(task->info) + 4, data_length);
 
-    swWorker_onTask(&serv->factory, task);
-
     int _end = htonl(0);
     SwooleG.main_reactor->write(SwooleG.main_reactor, conn->fd, (void *) &_end, sizeof(_end));
 
+    swWorker_onTask(&serv->factory, task);
+
     return SW_OK;
 }
 
@@ -288,7 +286,6 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
 #endif
 
     factory->last_from_id = task->info.from_id;
-    serv->last_session_id = task->info.fd;
     swWorker *worker = SwooleWG.worker;
     //worker busy
     worker->status = SW_WORKER_BUSY;
@@ -753,7 +750,7 @@ int swWorker_loop(swFactory *factory, int worker_id)
         swError("[Worker] create worker_reactor failed.");
         return SW_ERR;
     }
-    
+
     worker->status = SW_WORKER_IDLE;
 
     int pipe_worker = worker->pipe_worker;
