@@ -27,7 +27,7 @@ int swSystemTimer_init(int interval, int use_pipe)
 {
     swTimer *timer = &SwooleG.timer;
     timer->lasttime = interval;
-    
+
     if (use_pipe)
     {
         if (swPipeNotify_auto(&timer->pipe, 0, 0) < 0)
@@ -108,6 +108,10 @@ static int swSystemTimer_set(swTimer *timer, long new_interval)
     {
         return SW_OK;
     }
+    if (new_interval == 0)
+    {
+        new_interval = 1;
+    }
     current_interval = new_interval;
     return swSystemTimer_signal_set(timer, new_interval);
 }
