@@ -211,7 +211,7 @@ static Socket* client_coro_new(zval *object, int port)
 #ifdef SW_USE_OPENSSL
     if (type & SW_SOCK_SSL)
     {
-        cli->open_ssl = 1;
+        cli->open_ssl = true;
     }
 #endif
 
@@ -428,10 +428,19 @@ void php_swoole_client_coro_check_setting(Socket *cli, zval *zset TSRMLS_DC)
         swSocket_bind(cli->socket->fd, cli->type, bind_address, &bind_port);
     }
     /**
-     * TCP_NODELAY
+     * client: tcp_nodelay
      */
     if (php_swoole_array_get_value(vht, "open_tcp_nodelay", v))
     {
+        convert_to_boolean(v);
+        if (Z_BVAL_P(v))
+        {
+            goto _open_tcp_nodelay;
+        }
+    }
+    else
+    {
+        _open_tcp_nodelay:
         value = 1;
         if (setsockopt(cli->socket->fd, IPPROTO_TCP, TCP_NODELAY, &value, sizeof(value)) < 0)
         {
@@ -721,20 +730,11 @@ static PHP_METHOD(swoole_client_coro, connect)
 
     if (!cli->connect(host, port, sock_flag))
     {
-#ifdef SW_USE_OPENSSL
-        _error:
-#endif
         zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), SW_STRL("errCode")-1, cli->errCode);
         swoole_php_error(E_WARNING, "connect to server[%s:%d] failed. Error: %s[%d]", host, (int )port, cli->errMsg,
                 cli->errCode);
         RETURN_FALSE;
     }
-#ifdef SW_USE_OPENSSL
-    if (cli->open_ssl && !cli->ssl_handshake())
-    {
-        goto _error;
-    }
-#endif
     zend_update_property_bool(swoole_client_coro_class_entry_ptr, getThis(), SW_STRL("connected")-1, 1);
     RETURN_TRUE;
 }
@@ -844,9 +844,7 @@ static PHP_METHOD(swoole_client_coro, recvfrom)
     }
 
     zend_string *retval = zend_string_alloc(length + 1, 0);
-    char tmp_address[SW_IP_MAX_LENGTH];
-    int tmp_port;
-    ssize_t n_bytes = cli->recvfrom(retval->val, length, tmp_address, &tmp_port);
+    ssize_t n_bytes = cli->recvfrom(retval->val, length);
     if (n_bytes < 0)
     {
         zend_string_free(retval);
@@ -856,8 +854,8 @@ static PHP_METHOD(swoole_client_coro, recvfrom)
     {
         ZSTR_LEN(retval) = n_bytes;
         ZSTR_VAL(retval)[ZSTR_LEN(retval)] = '\0';
-        ZVAL_STRING(address, tmp_address);
-        ZVAL_LONG(port, tmp_port);
+        ZVAL_STRING(address, swConnection_get_ip(cli->socket));
+        ZVAL_LONG(port, swConnection_get_port(cli->socket));
         RETURN_STR(retval);
     }
 }
@@ -1161,7 +1159,7 @@ static PHP_METHOD(swoole_client_coro, enableSSL)
         swoole_php_fatal_error(E_WARNING, "SSL has been enabled.");
         RETURN_FALSE;
     }
-    cli->open_ssl = 1;
+    cli->open_ssl = true;
     zval *zset = sw_zend_read_property(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1 TSRMLS_CC);
     if (zset && !ZVAL_IS_NULL(zset))
     {
