@@ -44,9 +44,6 @@
 #define SW_USE_MONOTONIC_TIME
 //#define SW_USE_RINGBUFFER
 
-//#define SW_USE_TIMEWHEEL
-#define SW_TIMEWHEEL_SIZE          60
-
 //#define SW_DEBUG_REMOTE_OPEN
 #define SW_DEBUG_SERVER_HOST       "127.0.0.1"
 #define SW_DEBUG_SERVER_PORT       9999
