@@ -5,6 +5,17 @@
 extern "C" {
 #endif
 #include "coroutine.h"
+#include "zend_vm.h"
+#include "zend_closures.h"
+
+/* PHP 7.3 compatibility macro {{{*/
+#ifndef ZEND_CLOSURE_OBJECT
+# define ZEND_CLOSURE_OBJECT(func) (zend_object*)func->op_array.prototype
+#endif
+#ifndef GC_ADDREF
+# define GC_ADDREF(ref) ++GC_REFCOUNT(ref)
+# define GC_DELREF(ref) --GC_REFCOUNT(ref)
+#endif/*}}}*/
 
 #define SW_EX_CV_NUM(ex, n) (((zval ***)(((char *)(ex)) + ZEND_MM_ALIGNED_SIZE(sizeof(zend_execute_data)))) + n)
 #define SW_EX_CV(var) (*SW_EX_CV_NUM(execute_data, var))
@@ -60,8 +71,8 @@ typedef struct _php_context
     zval **current_coro_return_value_ptr_ptr;
     zval *current_coro_return_value_ptr;
     zval coro_params;
-    void (*onTimeout)(struct _php_context *cxt);
     void *private_data;
+    swTimer_node *timer;
     zval **current_eg_return_value_ptr_ptr;
     zend_execute_data *current_execute_data;
     zval *current_vm_stack_top;
@@ -86,8 +97,6 @@ typedef struct _coro_global
     zend_execute_data *origin_ex;
     coro_task *current_coro;
     zend_bool active;
-    coro_task *call_stack[SW_MAX_CORO_NESTING_LEVEL];
-    int call_stack_size;
     int error;
 } coro_global;
 
@@ -124,8 +133,9 @@ void sw_coro_yield();
 void sw_coro_close();
 int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_retval);
 void sw_coro_save(zval *return_value, php_context *sw_php_context);
-coro_task* sw_get_current_task();
+void sw_coro_set_stack_size(int stack_size);
 
+extern int swoole_coroutine_sleep(double msec);
 int php_swoole_add_timer_coro(int ms, int cli_fd, long *timeout_id, void* param, swLinkedList_node **node TSRMLS_DC);
 int php_swoole_clear_timer_coro(long id TSRMLS_DC);
 
