@@ -190,7 +190,7 @@ static user_opcode_handler_t ori_exit_handler = NULL;
 enum sw_exit_flags
 {
     SW_EXIT_IN_COROUTINE = 1 << 1,
-    SW_EXIT_IN_SERVER = 1<< 2
+    SW_EXIT_IN_SERVER = 1 << 2
 };
 
 static int coro_exit_handler(zend_execute_data *execute_data)
@@ -279,8 +279,12 @@ void swoole_coroutine_util_init(int module_number TSRMLS_DC)
     swoole_exit_exception_class_entry_ptr = zend_register_internal_class_ex(&swoole_exit_exception_ce, zend_exception_get_default());
     SWOOLE_DEFINE(EXIT_IN_COROUTINE);
     SWOOLE_DEFINE(EXIT_IN_SERVER);
-    ori_exit_handler = zend_get_user_opcode_handler(ZEND_EXIT);
-    zend_set_user_opcode_handler(ZEND_EXIT, coro_exit_handler);
+
+    if (SWOOLE_G(cli))
+    {
+        ori_exit_handler = zend_get_user_opcode_handler(ZEND_EXIT);
+        zend_set_user_opcode_handler(ZEND_EXIT, coro_exit_handler);
+    }
 }
 
 static PHP_METHOD(swoole_exit_exception, getFlags)
@@ -360,6 +364,7 @@ static PHP_METHOD(swoole_coroutine_util, set)
     {
         convert_to_long(v);
         COROG.stack_size = (uint32_t) Z_LVAL_P(v);
+        sw_coro_set_stack_size(COROG.stack_size);
     }
     if (php_swoole_array_get_value(vht, "log_level", v))
     {
@@ -479,6 +484,7 @@ static PHP_METHOD(swoole_coroutine_util, resume)
 static PHP_METHOD(swoole_coroutine_util, stats)
 {
     array_init(return_value);
+    sw_add_assoc_long_ex(return_value, ZEND_STRS("stack_size"), COROG.stack_size);
     sw_add_assoc_long_ex(return_value, ZEND_STRS("coroutine_num"), COROG.coro_num);
     sw_add_assoc_long_ex(return_value, ZEND_STRS("coroutine_peak_num"), COROG.peak_coro_num);
 }
@@ -488,23 +494,6 @@ static PHP_METHOD(swoole_coroutine_util, getuid)
     RETURN_LONG(sw_get_current_cid());
 }
 
-static void php_coroutine_sleep_timeout(swTimer *timer, swTimer_node *tnode)
-{
-    zval *retval = NULL;
-    zval *result = NULL;
-    SW_MAKE_STD_ZVAL(result);
-    ZVAL_BOOL(result, 1);
-
-    php_context *context = (php_context *) tnode->data;
-    int ret = coro_resume(context, result, &retval);
-    if (ret == CORO_END && retval)
-    {
-        sw_zval_ptr_dtor(&retval);
-    }
-    sw_zval_ptr_dtor(&result);
-    efree(context);
-}
-
 int php_coroutine_reactor_can_exit(swReactor *reactor)
 {
     return COROG.coro_num == 0;
@@ -538,19 +527,11 @@ static PHP_METHOD(swoole_coroutine_util, sleep)
         return;
     }
 
-    php_context *context = emalloc(sizeof(php_context));
-    context->onTimeout = NULL;
-    context->state = SW_CORO_CONTEXT_RUNNING;
-
     php_swoole_check_reactor();
     php_swoole_check_timer(ms);
-    if (SwooleG.timer.add(&SwooleG.timer, ms, 0, context, php_coroutine_sleep_timeout) == NULL)
-    {
-        RETURN_FALSE;
-    }
 
-    coro_save(context);
-    coro_yield();
+    swoole_coroutine_sleep(seconds);
+    RETURN_TRUE;
 }
 
 static void aio_onReadCompleted(swAio_event *event)
@@ -800,7 +781,6 @@ static void co_socket_read(int fd, zend_long length, INTERNAL_FUNCTION_PARAMETER
     sock->buf = zend_string_alloc(length + 1, 0);
     sock->nbytes = length <= 0 ? SW_BUFFER_SIZE_STD : length;
 
-    sock->context.onTimeout = NULL;
     sock->context.state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(&sock->context);
@@ -837,7 +817,6 @@ static void co_socket_write(int fd, char* str, size_t l_str, INTERNAL_FUNCTION_P
 
     php_context *context = &sock->context;
     context->state = SW_CORO_CONTEXT_RUNNING;
-    context->onTimeout = NULL;
     context->private_data = str;
 
     sock->nbytes = l_str;
@@ -913,6 +892,7 @@ static PHP_METHOD(swoole_coroutine_util, fread)
     ev.flags = 0;
     ev.type = SW_AIO_READ;
     ev.object = context;
+    ev.handler = swAio_handler_read;
     ev.callback = aio_onReadCompleted;
     ev.fd = fd;
     ev.offset = _seek;
@@ -932,7 +912,6 @@ static PHP_METHOD(swoole_coroutine_util, fread)
         RETURN_FALSE;
     }
 
-    context->onTimeout = NULL;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(context);
@@ -994,6 +973,7 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
     ev.type = SW_AIO_STREAM_GET_LINE;
     ev.object = context;
     ev.callback = aio_onStreamGetLineCompleted;
+    ev.handler = swAio_handler_stream_get_line;
     ev.fd = fd;
     ev.offset = stream->readpos;
     ev.req = (void *) (long) stream->writepos;
@@ -1014,7 +994,6 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
     }
 
     context->coro_params = *handle;
-    context->onTimeout = NULL;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(context);
@@ -1084,6 +1063,7 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
     ev.flags = 0;
     ev.type = SW_AIO_WRITE;
     ev.object = context;
+    ev.handler = swAio_handler_write;
     ev.callback = aio_onWriteCompleted;
     ev.fd = fd;
     ev.offset = _seek;
@@ -1099,7 +1079,6 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
         RETURN_FALSE;
     }
 
-    context->onTimeout = NULL;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(context);
@@ -1131,6 +1110,7 @@ static PHP_METHOD(swoole_coroutine_util, readFile)
 
     ev.type = SW_AIO_READ_FILE;
     ev.object = context;
+    ev.handler = swAio_handler_read_file;
     ev.callback = aio_onReadFileCompleted;
     ev.req = estrndup(filename, l_filename);
 
@@ -1149,7 +1129,6 @@ static PHP_METHOD(swoole_coroutine_util, readFile)
         RETURN_FALSE;
     }
 
-    context->onTimeout = NULL;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(context);
@@ -1190,6 +1169,7 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
 
     ev.type = SW_AIO_WRITE_FILE;
     ev.object = context;
+    ev.handler = swAio_handler_write_file;
     ev.callback = aio_onWriteFileCompleted;
     ev.req = estrndup(filename, l_filename);
     ev.flags = O_CREAT | O_WRONLY;
@@ -1218,7 +1198,6 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
         RETURN_FALSE;
     }
 
-    context->onTimeout = NULL;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
     coro_save(context);
@@ -1366,6 +1345,7 @@ static PHP_METHOD(swoole_coroutine_util, gethostbyname)
     ev.flags = family;
     ev.type = SW_AIO_GETHOSTBYNAME;
     ev.object = sw_current_context;
+    ev.handler = swAio_handler_gethostbyname;
     ev.callback = coro_dns_onResolveCompleted;
 
     php_swoole_check_aio();
@@ -1420,6 +1400,7 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
 
     ev.type = SW_AIO_GETADDRINFO;
     ev.object = sw_current_context;
+    ev.handler = swAio_handler_getaddrinfo;
     ev.callback = coro_dns_onGetaddrinfoCompleted;
     ev.req = req;
 
@@ -1470,7 +1451,7 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
     }
     else
     {
-        coro_task *task = (coro_task *) coroutine_get_ptr_by_cid(cid);
+        coro_task *task = (coro_task *) coroutine_get_task_by_cid(cid);
         if (task == NULL)
         {
             RETURN_FALSE;
