@@ -582,13 +582,6 @@ static PHP_METHOD(swoole_process, alarm)
         RETURN_FALSE;
     }
 
-    struct timeval now;
-    if (gettimeofday(&now, NULL) < 0)
-    {
-        swoole_php_error(E_WARNING, "gettimeofday() failed. Error: %s[%d]", strerror(errno), errno);
-        RETURN_FALSE;
-    }
-
     struct itimerval timer_set;
     bzero(&timer_set, sizeof(timer_set));
 
