@@ -8,9 +8,11 @@ assert_options(ASSERT_WARNING, 1);
 assert_options(ASSERT_BAIL, 0);
 assert_options(ASSERT_QUIET_EVAL, 0);
 
-if (method_exists('co', 'set')) {
-    co::set([
-        'log_level' => SWOOLE_LOG_INFO,
-        'trace_flags' => 0
-    ]);
-}
\ No newline at end of file
+if (empty(getenv('SWOOLE_DEBUG'))) {
+    if (method_exists('co', 'set')) {
+        co::set([
+            'log_level' => SWOOLE_LOG_INFO,
+            'trace_flags' => 0,
+        ]);
+    }
+}
