@@ -1,5 +1,5 @@
 <?php
-require_once dirname(dirname(__DIR__)).'/config.php';
+require_once dirname(dirname(__DIR__)).'/bootstrap.php';
 
 class TcpServer
 {
